import ldap3
import logging
import re

from .colors import c_green
from .colors import c_orange
from .colors import c_red
from .colors import c_white_on_red
from .colors import c_cyan

from .exceptions import LsaException

from .logging import log_error
from .logging import log_warning
from .logging import log_info
from .logging import log_success

from .utils import str_human_date
from .utils import str_functionality_level
from .utils import list_uac_colored_flags
from .utils import str_samaccounttype
from .utils import str_object_type
from .utils import list_trust_type
from .utils import list_trust_direction
from .utils import list_trust_attributes
from .utils import convert_sid_to_string


class LdapsearchAd:

    hostname = None
    port = None
    domain = None
    username = None
    password = None
    server = None
    connection = None

    last_errors: list = []

    def __init__(
        self,
        hostname,
        # default ldaps port if not specified
        port=636,
        ssl=False,
        domain=None,
        username=None,
        password=None,
        hashes=None,
    ):
        self.hostname = hostname
        self.port = port
        self.domain = domain
        self.username = username
        self.password = password
        self.hashes = hashes
        if self.hashes is not None:
            try:
                self.lmhash, self.nthash = self.hashes.split(":")
            except ValueError as ve:
                print("Error: The hash need to be in the following format:")
                print("aad3b435b51404eeaad3b435b51404ee:382c7bf814461d8d685cf7a7a06c8c8f")
                print("")
                raise ve
        try:
            self.server = ldap3.Server(self.hostname, self.port, use_ssl=ssl, get_info="ALL")
            if self.domain and self.username:
                if self.password:
                    self.connection = ldap3.Connection(
                        self.server,
                        user=f"{self.domain}\\{self.username}",
                        password=self.password,
                        authentication="NTLM",
                        auto_bind=True,
                        read_only=True,
                    )
                elif self.hashes is not None:
                    if self.lmhash == "":
                        self.lmhash = "aad3b435b51404eeaad3b435b51404ee"
                    self.connection = ldap3.Connection(
                        self.server,
                        user=f"{self.domain}\\{self.username}",
                        password=self.lmhash + ":" + self.nthash,
                        authentication=ldap3.NTLM,
                        auto_bind=True,
                        read_only=True,
                    )
                else:
                    raise LsaException("Need either password or hash to try to authenticate.")
            else:
                self.connection = ldap3.Connection(self.server, auto_bind=True, read_only=True)
        except ldap3.core.exceptions.LDAPSocketOpenError as ldapsoe:
            self.last_errors.append(f"Unable to connect to {self.hostname}.")
            raise ldapsoe
        except ldap3.core.exceptions.LDAPBindError as ldapbe:
            self.last_errors.append(f"Bind Error: {ldapbe}")
            raise ldapbe

    def is_connected(self):
        return self.server is not None and self.connection is not None

    def is_authenticated(self):
        return self.server is not None and self.connection is not None and self.connection.authentication != ldap3.ANONYMOUS

    def infos(self):
        if not self.is_connected():
            raise LsaException("Need to be connected before reading server's infos.")
        log_info(f'Forest functionality level = {str_functionality_level(self.server.info.other["forestFunctionality"][0])}')
        log_info(f'Domain functionality level = {str_functionality_level(self.server.info.other["domainFunctionality"][0])}')
        log_info(
            f'Domain controller functionality level = {str_functionality_level(self.server.info.other["domainControllerFunctionality"][0])}'
        )
        log_info(f'rootDomainNamingContext = {self.server.info.other["rootDomainNamingContext"][0]}')
        log_info(f'defaultNamingContext = {self.server.info.other["defaultNamingContext"][0]}')
        log_info(f'ldapServiceName = {self.server.info.other["ldapServiceName"][0]}')
        log_info(f"naming_contexts = {self.server.info.naming_contexts}")

    def whoami(self):
        if not self.is_connected():
            raise LsaException("Need to be connected before trying to check who you are.")
        if self.connection.extend.standard.who_am_i():
            log_success(self.connection.extend.standard.who_am_i())
        else:
            log_error("<not_connected>")

    def search(self, search_filter, attributes="*", size_limit=100, page_size=1000, search_base=None):
        """Warning: returns a generator!
        Thus, it is not backward compatible with the old_search
        Arguments:
        * search_filter = '(samaccountname=adm*)'
        * attributes = ['cn', 'givenName'] or '*' for all attributes
        Return value:
        * a generator of CaseInsensitiveDict"""

        if not self.is_authenticated():
            raise LsaException("Need to be authenticated before trying to search for something.")
        if isinstance(size_limit, str):
            try:
                size_limit = int(size_limit)
            except ValueError:
                raise LsaException("size_limit (-z) should be a valid integer")
        nb_entries = 0
        if search_base:
            base_dn = search_base
        else:
            base_dn = self.server.info.other.get("defaultNamingContext")[0]
        try:
            self.connection.search(
                search_base=base_dn,
                search_filter=search_filter,
                search_scope=ldap3.SUBTREE,
                attributes=attributes,
                paged_size=page_size,
                size_limit=size_limit,
            )
        except ldap3.core.exceptions.LDAPSocketOpenError:
            raise LsaException("LDAPSocketOpenError during search. Maybe the search_base is incorrect?")
        for entry in self.connection.response:
            if entry["type"] == "searchResEntry":
                nb_entries += 1
                yield entry["attributes"]
        if "controls" in self.connection.result:
            cookie = self.connection.result["controls"]["1.2.840.113556.1.4.319"]["value"]["cookie"]
            size_limit -= nb_entries
            nb_entries = 0
        else:
            size_limit = 0
            cookie = None
        while size_limit > 0 and cookie:
            self.connection.search(
                search_base=base_dn,
                search_filter=search_filter,
                search_scope=ldap3.SUBTREE,
                attributes=attributes,
                paged_size=page_size,
                size_limit=size_limit,
                paged_cookie=cookie,
            )
            for entry in self.connection.response:
                if entry["type"] == "searchResEntry":
                    nb_entries += 1
                    yield entry["attributes"]
            if "controls" in self.connection.result:
                cookie = self.connection.result["controls"]["1.2.840.113556.1.4.319"]["value"]["cookie"]
                size_limit -= nb_entries
                nb_entries = 0
            else:
                size_limit = 0
                cookie = None

    def __list_groups(self, entry):
        """Return a list containing the CN of each group the parameter is member of."""
        if "memberOf" not in entry:
            return ["memberOf attribute not found"]
        groups = []
        # dirty patch because "memberOf.value" return a string if there is only one group
        # and a list of string if there is more than one group
        logging.debug(f'type of memberOf = {type(entry["memberOf"])}')
        if isinstance(entry["memberOf"], str):
            groups_raw = [entry["memberOf"]]
        else:
            groups_raw = entry["memberOf"]
        for group in groups_raw:
            group_cn = re.search("CN=([^,]*),", group).group(1)
            if re.search("^(enterprise admins)$", group_cn, re.IGNORECASE):
                groups.append(c_red(group_cn))
            elif re.search("^(domain admins|admins du domaine)$", group_cn, re.IGNORECASE):
                groups.append(c_red(group_cn))
            elif re.search("^(administrators|administrateurs)$", group_cn, re.IGNORECASE):
                groups.append(c_red(group_cn))
            elif re.search("admin", group_cn, re.IGNORECASE):
                groups.append(c_orange(group_cn))
            else:
                groups.append(group_cn)
        return groups

    def __print_user_details(self, user, tab=""):
        """Print info of a user (samacountname and userAccountControl)."""
        log_info(f'{tab}{user["samAccountName"]}')
        log_info(f"{tab}|__ type: {str_object_type(user)}")
        if "displayName" in user:
            log_info(f'{tab}|__ displayName = {user["displayName"]}')
        if "description" in user:
            log_info(f'{tab}|__ description = {user["description"]}')
        if "adminCount" in user:
            if user["admincount"] == 1:
                log_success(f'{tab}|__ {c_red("The adminCount is set to 1")}')
            elif user["admincount"] == 0:
                log_info(f'{tab}|__ {"adminCount = 0"}')
            else:
                log_error(f'{tab}|__ Unknown value for adminCount: {user["admincount"]}')
        if "userAccountControl" in user:
            log_info(f'{tab}|__ userAccountControl = {", ".join(list_uac_colored_flags(user["userAccountControl"]))}')
        log_info(f'{tab}|__ sAMAccountType = {str_samaccounttype(user["samaccounttype"])}')
        if "memberOf" in user:
            log_info(f'{tab}|__ memberOf = {", ".join(self.__list_groups(user))}')

    def __print_user_brief(self, user, tab=""):
        """Print info of a user on a single line (samacountname and userAccountControl)."""
        if str_object_type(user) != "user":
            if "foreignSecurityPrincipal" in user["objectclass"]:
                log_info(
                    f'{tab}{user["sAMAccountName"]} (objectclass = foreignSecurityPrincipal, probably a user from another domain. Please investigate)'
                )
            else:
                log_warning(f'{tab}{user["sAMAccountName"]} ({c_orange(str_object_type(user))})')
        else:
            uac_flags = list_uac_colored_flags(user["userAccountControl"])
            uac_flags.remove("NORMAL_ACCOUNT")
            if uac_flags:
                log_success(f'{tab}{user["sAMAccountName"]} ({", ".join(uac_flags)})')
            else:
                log_info(f'{tab}{user["sAMAccountName"]}')

    def __print_group_brief(self, group, tab=""):
        """Print info of a group on a single line (samacountname and description)."""
        if isinstance(group["description"], list):
            description = " - ".join(group["description"])
        else:
            description = str(group["description"])
        if "member" in group:
            nb_members = len(group["member"])
        else:
            nb_members = 0
        log_info(f'{tab}{group["sAMAccountName"]}§{str_object_type(group)}§{description}§{nb_members}')

    def print_users(self, search_filter, attributes="*", size_limit=100):
        """Method to pretty print a set a users attributes."""
        r_search = self.search(search_filter, attributes, size_limit=size_limit)
        for user in r_search:
            self.__print_user_details(user)

    def print_users_list(self, search_filter, attributes="*", size_limit=100):
        """Method to pretty list print a set a users attributes."""
        r_search = self.search(search_filter, attributes, size_limit=size_limit)
        for user in r_search:
            self.__print_user_brief(user)

    def print_member_of(self, group_cn, size_limit=100):
        """Print the list of users who a member of a specific group.
        Also use the nested groups"""
        search_filter = f"(CN={group_cn})"
        # Get the exact distinguishedName of the requested group
        # needed to perform a recursive search of members of members of members ...
        targeted_groups = self.search(search_filter, ["distinguishedName", "cn"], size_limit=100)
        # should be only one...
        for targeted_group in targeted_groups:
            group_dn = targeted_group["distinguishedName"]
            log_info(f'All members of group "{targeted_group["cn"]}":')
            # from this distinguishedName, find all members recursively
            search_filter = f"(&(memberOf:1.2.840.113556.1.4.1941:={group_dn})(!(objectClass=group)))"
            attributes = [
                "objectClass",
                "name",
                "userAccountControl",
                "sAMAccountName",
                "sAMAccountType",
            ]
            users = self.search(search_filter, attributes, size_limit=size_limit)
            for user in users:
                self.__print_user_brief(user, "    ")

    def print_search_foreign_security_principals(self, size_limit=100):
        """Print the list of foreign security principals who are members of
        domain local groups in the current forest"""
        search_filter = "(objectclass=foreignSecurityPrincipal)"
        attributes = ["name", "objectSid", "distinguishedName", "objectClass"]
        fsp = self.search(search_filter, attributes, size_limit=size_limit)
        for sp in fsp:
            log_info(f'name = {sp["name"]}')
            log_info(f'|__ objectSid = {sp["objectSid"]}')
            log_info(f'|__ distinguishedName = {sp["distinguishedName"]}')
            log_info(f'|__ objectClass = {sp["objectClass"]}')

    def print_user_of(self, search_filter, size_limit=100):
        """Print the list of groups whom a user is member of.
        Also use the nested groups"""
        # Get the exact distinguishedName of the requested user
        # needed to perform a recursive search of his groups ...
        targeted_users = self.search(search_filter, ["distinguishedName", "cn"], size_limit=100)
        # should be only one...
        for targeted_user in targeted_users:
            user_dn = targeted_user["distinguishedName"]
            log_info(f'All groups of user "{targeted_user["cn"]}":')
            # from this distinguishedName, find all members recursively
            search_filter = f"(member:1.2.840.113556.1.4.1941:={user_dn})"
            attributes = [
                "objectClass",
                "description",
                "cn",
                "member",
                "sAMAccountName",
                "sAMAccountType",
            ]
            groups = self.search(search_filter, attributes, size_limit=size_limit)
            for group in groups:
                self.__print_group_brief(group, "    ")

    def print_trusts(self):
        """Method to get infos about trusts."""
        for trust in self.search("(objectClass=trustedDomain)"):
            log_info(f'+ {trust["name"]} ({trust["flatName"]})')
            log_info(f'|__ trustAttributes = {list_trust_attributes(trust["trustAttributes"])}')
            log_info(f'|__ trustDirection = {list_trust_direction(trust["trustDirection"])}')
            log_info(f'|__ trustType = {list_trust_type(trust["trustType"])}')
            log_info(f'|__ trustPartner = {trust["trustPartner"]}')
            if "securityIdentifier" in trust:
                log_info(
                    f'|__ securityIdentifier = {ldap3.protocol.formatters.formatters.format_sid(trust["securityIdentifier"])}'
                )
            log_info(f'|__ whenCreated = {trust["whenCreated"]}')
            log_info(f'|__ whenChanged = {trust["whenChanged"]}')

    def __print_default_pass_pol(self, pass_pol):
        """Print info about the default password policy."""
        log_info("Default password policy:")
        min_pass_len = pass_pol["minPwdLength"]
        # Password length
        if min_pass_len < 8:
            log_info(f"|__ Minimum password length = {c_red(min_pass_len)}")
        elif min_pass_len < 12:
            log_info(f"|__ Minimum password length = {c_orange(min_pass_len)}")
        else:
            log_info(f"|__ Minimum password length = {c_green(min_pass_len)}")
        # Password properties as described here: https://ldapwiki.com/wiki/PwdProperties
        pass_properties = pass_pol["pwdProperties"]
        if pass_properties & 1 > 0:
            log_info(f'|__ Password complexity = {c_green("Enabled")}')
        else:
            log_info(f'|__ Password complexity = {c_red("Disabled")}')
        # Lockout settings
        if pass_pol["lockoutThreshold"] == 0:
            log_success(f'|__ Lockout threshold = {c_white_on_red("Disabled")}')
        else:
            if pass_pol["lockoutThreshold"] > 5:
                log_success(f'|__ Lockout threshold = {pass_pol["lockoutThreshold"]}')
            else:
                log_info(f'|__ Lockout threshold = {pass_pol["lockoutThreshold"]}')
            log_info(f'|__ Lockout duration = {str_human_date(pass_pol["lockoutDuration"])}')
            log_info(f'|__ Lockout observation window = {str_human_date(pass_pol["lockOutObservationWindow"])}')
        # Password history length
        if pass_pol["pwdHistoryLength"] > 0:
            log_success(f'|__ Password history length = {pass_pol["pwdHistoryLength"]}')
        else:
            log_info(f'|__ Password history length = {pass_pol["pwdHistoryLength"]}')
        # Password min and max age
        log_info(f'|__ Max password age = {str_human_date(pass_pol["maxPwdAge"])}')
        log_info(f'|__ Min password age = {str_human_date(pass_pol["minPwdAge"])}')

    def __print_pass_pol(self, pass_pol):
        """Print info about a Fine-Grained Password Policy."""
        log_info(f'Fined grained password policy found: {c_cyan(pass_pol["cn"])}')
        log_info(f'|__ Password settings precedence = {pass_pol["msDS-PasswordSettingsPrecedence"]}')
        pass_len = pass_pol["msDS-MinimumPasswordLength"]
        # Password length
        if pass_len < 8:
            log_info(f"|__ Minimum password length = {c_red(pass_len)}")
        elif pass_len < 12:
            log_info(f"|__ Minimum password length = {c_orange(pass_len)}")
        else:
            log_info(f"|__ Minimum password length = {c_green(pass_len)}")
        # Password complexity
        if pass_pol["msDS-PasswordComplexityEnabled"]:
            log_info(f'|__ Password complexity enabled = {c_green("Enabled")}')
        else:
            log_info(f'|__ Password complexity enabled = {c_red("Disabled")}')
        # Password reversible encryption?
        if pass_pol["msDS-PasswordReversibleEncryptionEnabled"]:
            log_info(
                f'|__ Password reversible encryption enabled = {c_white_on_red(pass_pol["msDS-PasswordReversibleEncryptionEnabled"])}'
            )
        else:
            log_info(f'|__ Password reversible encryption enabled = {pass_pol["msDS-PasswordReversibleEncryptionEnabled"]}')
        # Lockout settings
        if pass_pol["msDS-LockoutThreshold"] == 0:
            log_success(f'|__ Lockout threshold = {c_white_on_red("Disabled")}')
        else:
            log_info(f'|__ Lockout threshold = {pass_pol["msDS-LockoutThreshold"]}')
            log_info(f'|__ Lockout duration = {str_human_date(pass_pol["msDS-LockoutDuration"])}')
            log_info(f'|__ Lockout observation window = {str_human_date(pass_pol["msDS-LockoutObservationWindow"])}')
        # Password history length
        if pass_pol["msDS-PasswordHistoryLength"] > 0:
            log_success(f'|__ Password history length = {pass_pol["msDS-PasswordHistoryLength"]}')
        else:
            log_info(f'|__ Password history length = {pass_pol["msDS-PasswordHistoryLength"]}')
        # Password min and max age
        log_info(f'|__ Max password age = {str_human_date(pass_pol["msDS-MaximumPasswordAge"])}')
        log_info(f'|__ Min password age = {str_human_date(pass_pol["msDS-MinimumPasswordAge"])}')
        log_info(f'|__ PSO applies to = {pass_pol["msDS-PSOAppliesTo"]}')

    def print_pass_pols(self):
        """Main function to get info about password policies."""
        # get default password policy
        default_pps = self.search("(objectClass=domainDNS)", size_limit=5)
        nb_default_pps = 0
        for default_pp in default_pps:
            nb_default_pps += 1
            self.__print_default_pass_pol(default_pp)
        if nb_default_pps > 1:
            raise LsaException(
                'More than one "default password policy" found. Should not happened. Please investigate and correct the script.'
            )
        # get Fine Grained Password Policies
        fgpps = self.search("(objectClass=MsDS-PasswordSettings)", size_limit=100)
        nb_fgpps = 0
        for fgpp in fgpps:
            nb_fgpps += 1
            self.__print_pass_pol(fgpp)
        if nb_fgpps <= 0:
            log_info("No fine grained password policy found (high privileges are required).")

    def print_admins(self, size_limit=100):
        """Method to get a list of members of the "admin" group."""
        english_groups = "(CN=Administrators)(CN=Domain Admins)(CN=Enterprise Admins)"
        french_groups = "(CN=Administrateurs)(CN=Admins du domaine)(CN=Administrateurs de l’entreprise)"
        search_filter = f"(|{english_groups}{french_groups})"
        # Get the exact distinguishedName of the "admin" group
        # needed to perform a recursive search of members of members of members ...
        admin_groups = self.search(search_filter, ["distinguishedName", "cn"], size_limit=10)
        for admin_group in admin_groups:
            admins_dn = admin_group["distinguishedName"]
            log_info(f'All members of group "{admin_group["cn"]}":')
            # from this distinguishedName, find all members recursively
            search_filter = f"(&(memberOf:1.2.840.113556.1.4.1941:={admins_dn})(!(objectClass=group)))"
            attributes = [
                "objectClass",
                "name",
                "userAccountControl",
                "sAMAccountName",
                "sAMAccountType",
            ]
            admins = self.search(search_filter, attributes, size_limit=size_limit)
            for admin in admins:
                self.__print_user_brief(admin, "    ")

    def __print_user_with_spn(self, user):
        spns = ", ".join(user["servicePrincipalName"])
        log_success(f"{user['sAMAccountName']}: {spns}")

    def print_kerberoast(self):
        """Method to get infos about kerberoastable users.
        Log its sAMAccountName and servicePrincipalName"""
        search_filter = "(&(objectClass=user)(servicePrincipalName=*)(!(objectClass=computer))(!(cn=krbtgt))(!(userAccountControl:1.2.840.113556.1.4.803:=2)))"
        search_attributes = ["cn", "samaccountname", "serviceprincipalname"]
        for kerberoastable_user in self.search(search_filter, search_attributes):
            self.__print_user_with_spn(kerberoastable_user)

    def get_kerberoast(self):
        """Method to get infos about kerberoastable users.
        Log its sAMAccountName and servicePrincipalName"""
        search_filter = "(&(objectClass=user)(servicePrincipalName=*)(!(objectClass=computer))(!(cn=krbtgt))(!(userAccountControl:1.2.840.113556.1.4.803:=2)))"
        search_attributes = ["cn", "samaccountname", "serviceprincipalname"]
        return self.search(search_filter, search_attributes)

    def print_asreqroast(self):
        """Method to get all accounts that are vulnerable to ASREPRoast.
        Filter based on https://www.tarlogic.com/en/blog/how-to-attack-kerberos/"""
        search_filter = "(&(samAccountType=805306368)(userAccountControl:1.2.840.113556.1.4.803:=4194304))"
        search_attributes = ["cn", "samaccountname"]
        for asreqroastuser in self.search(search_filter, search_attributes):
            log_success(f"{asreqroastuser}")

    def print_search_spn(self, search_filter, size_limit=100):
        """Method to find services registered in the AD."""
        if not re.search("serviceprincipalname", search_filter, re.IGNORECASE):
            search_filter = f"(servicePrincipalName={search_filter}*)"
        search_attributes = ["cn", "samaccountname", "serviceprincipalname"]
        for spn_user in self.search(search_filter, search_attributes, size_limit=size_limit):
            self.__print_user_with_spn(spn_user)

    def print_lastpwchangekrbtgt(self):
        """Method to retreive the last time the password for krbtgt was reset."""
        search_filter = "(cn=krbtgt)"
        search_attributes = ["pwdLastSet"]
        for krbtgt in self.search(search_filter, search_attributes):
            # when_changed = krbtgt['pwdLastSet'].replace(microsecond=0).isoformat()
            # this one is similar but easier to read
            when_changed = krbtgt["pwdLastSet"].strftime("%Y-%m-%d %H:%M:%S")
            log_info(f"krbtgt password changed at {when_changed}")

    def print_search_delegation(self):
        """Method to retreive accounts with TRUSTED_FOR_DELEGATION set in userAccountControl"""
        search_filter = "(userAccountControl:1.2.840.113556.1.4.803:=524288)"
        search_attributes = ["samaccountname"]
        for account_trusted_for_delegation in self.search(search_filter, search_attributes):
            log_success(f"{account_trusted_for_delegation['sAMAccountName']}")

    def print_creator_sid(self):
        """Main function to get info about createsid from ms-ds-creatorsid."""
        # get all createsid without parse
        search_filter = "(ms-ds-creatorsid=*)"
        search_attributes = ["sAMAccountName", "mS-DS-CreatorSID"]
        for asreqroastuser in self.search(search_filter, search_attributes):
            log_success(f"{asreqroastuser}")
            if asreqroastuser["mS-DS-CreatorSID"] != "":
                sid_str = convert_sid_to_string(asreqroastuser["mS-DS-CreatorSID"])
            log_info(sid_str)
            # parse objectSid
            search_filter_get_name = f"(objectSid={sid_str})"
            search_attributes_get_name = ["sAMAccountName"]
            asreqroastuser_get_name_obj = self.search(search_filter_get_name, search_attributes_get_name)
            if asreqroastuser_get_name_obj:
                for asreqroastuser_get_name in asreqroastuser_get_name_obj:
                    if asreqroastuser_get_name["sAMAccountName"] is not None:
                        log_info(f'CreatorSID: {asreqroastuser_get_name["sAMAccountName"]}')
            else:
                log_error("Maybe already deleted")



from logging import info
from logging import error
from logging import getLogger
from logging import StreamHandler
from logging import Formatter
from logging import INFO as LOGGING_INFO

from .colors import c_red
from .colors import c_orange
from .colors import c_blue
from .colors import c_green

from sys import stdout


def log_title(title, level=2):
    heading = "#" * level
    info(f"\x1b[1;37;40m{heading} {title} {heading}\x1b[0m")


def log_error(message):
    error(f'{c_red("[-]")} {message}')


def log_warning(message):
    info(f'{c_orange("[!]")} {message}')


def log_info(message):
    info(f'{c_blue("[+]")} {message}')


def log_success(message):
    info(f'{c_green("[*]")} {message}')


# Configure default logging to stdout
logger = getLogger()
handler = StreamHandler(stdout)
logger.setLevel(LOGGING_INFO)
formatter = Formatter(fmt="%(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)



#!/usr/bin/env python3

import argparse
import logging

from sys import exit
from sys import stdout

from os import path as os_path

from ldapsearchad import LdapsearchAd
from ldapsearchad import version as ldapsearchad_version

from ldapsearchad.exceptions import LsaException

from ldapsearchad.logging import log_title
from ldapsearchad.logging import log_error
from ldapsearchad.logging import log_info

from ldapsearchad.utils import list_uac_flags
from ldapsearchad.utils import str_samaccounttype


def main():
    # Parse arguments
    arg_parser = argparse.ArgumentParser(description="Active Directory LDAP Enumerator")
    arg_parser.add_argument("-l", "--server", dest="ldap_server", help="IP address of the LDAP server.")
    arg_parser.add_argument(
        "-n",
        "--port",
        dest="server_port_number",
        type=int,
        help="Port number to request (389 (LDAP), 636 (LDAP over SSL), 3268 (GC), 3269 (GC over SSL)).",
    )
    arg_parser.add_argument(
        "-ssl",
        "--ssl",
        dest="ssl",
        action="store_true",
        help="Force an SSL connection?.",
    )
    arg_parser.add_argument(
        "-t",
        "--type",
        dest="request_type",
        help="Request type: info, whoami, search, csv, trusts,\
        pass-pols, admins, show-user, show-user-list, kerberoast, search-spn, asreproast, goldenticket,\
        search-delegation, createsid, search-foreign-security-principals, all",
    )
    arg_parser.add_argument(
        "-d",
        "--domain",
        dest="domain",
        help='Authentication account\'s FQDN. Example: "contoso.local".',
    )
    arg_parser.add_argument("-u", "--username", dest="username", help="Authentication account's username.")
    arg_parser.add_argument("-p", "--password", dest="password", help="Authentication account's password.")
    arg_parser.add_argument("-H", "-hashes", dest="hashes", help="NTLM hashes, format is LMHASH:NTHASH")
    arg_parser.add_argument(
        "-s",
        "--search-filter",
        dest="search_filter",
        help="Search filter (use LDAP format and don't forget parenthesis).",
    )
    arg_parser.add_argument(
        "-b",
        "--search-base",
        dest="search_base",
        help="Search base to use as the starting point for the search instead of the default. (example: 'CN=Configuration,DC=corp,DC=local')",
    )
    arg_parser.add_argument(
        "search_attributes",
        default="*",
        nargs="*",
        help="LDAP attributes to look for (default is all).",
    )
    arg_parser.add_argument(
        "-z",
        "--size_limit",
        dest="size_limit",
        default=100,
        help="Size limit (default is 100).",
    )
    arg_parser.add_argument(
        "-o",
        "--output",
        dest="output_file",
        help="Write results in specified file too.",
    )
    arg_parser.add_argument(
        "-v",
        "--verbose",
        dest="verbosity",
        help="Turn on debug mode",
        action="store_true",
    )
    arg_parser.add_argument(
        "--version",
        dest="ask_for_version",
        help="Show version and exit",
        action="store_true",
    )
    args = arg_parser.parse_args()

    if args.ask_for_version:
        print(f"ldapsearchad v{ldapsearchad_version()}")
        exit(0)

    # if the version is not asked, we should have at least a target and an action
    if args.ldap_server is None or args.request_type is None:
        arg_parser.error("-l/--server and -t/--type are mandatory arguments.")

    # Set mandatory arguments for each request_type
    mandatory_arguments = {}
    mandatory_arguments["info"] = []
    mandatory_arguments["whoami"] = ["domain", "username"]
    mandatory_arguments["search"] = ["domain", "username", "search_filter"]
    mandatory_arguments["csv"] = [
        "domain",
        "username",
        "search_filter",
        "search_attributes",
    ]
    mandatory_arguments["show-user"] = ["domain", "username", "search_filter"]
    mandatory_arguments["show-user-list"] = ["domain", "username", "search_filter"]
    mandatory_arguments["member-of"] = ["domain", "username", "search_filter"]
    mandatory_arguments["user-of"] = ["domain", "username", "search_filter"]
    mandatory_arguments["trusts"] = ["domain", "username"]
    mandatory_arguments["pass-pols"] = ["domain", "username"]
    mandatory_arguments["admins"] = ["domain", "username"]
    mandatory_arguments["kerberoast"] = ["domain", "username"]
    mandatory_arguments["asreproast"] = ["domain", "username"]
    mandatory_arguments["search-spn"] = ["domain", "username", "search_filter"]
    mandatory_arguments["goldenticket"] = ["domain", "username"]
    mandatory_arguments["search-delegation"] = ["domain", "username"]
    mandatory_arguments["createsid"] = ["domain", "username"]
    mandatory_arguments["search-foreign-security-principals"] = ["domain", "username"]
    mandatory_arguments["all"] = ["domain", "username"]
    actions = [i.strip() for i in args.request_type.split(",")]
    for action in actions:
        if action not in mandatory_arguments.keys():
            arg_parser.error(f'request type must be one of: {", ".join(mandatory_arguments.keys())}.')
        for mandatory_argument in mandatory_arguments[action]:
            if vars(args)[mandatory_argument] is None:
                arg_parser.error(f"{mandatory_argument} argument is mandatory with request type = {action}")
    # special verification for CSV export
    if "csv" in actions and args.search_attributes == "*":
        arg_parser.error("Must specify attributes for the CSV export.")

    # Configure logging to stdout
    logger = logging.getLogger()
    logger.removeHandler(logger.handlers[0])
    handler = logging.StreamHandler(stdout)
    if args.verbosity:
        logger.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            fmt="%(asctime)-19s %(levelname)-8s %(message)s",
            datefmt="%Y-%m-%d_%H:%M:%S",
        )
    else:
        logger.setLevel(logging.INFO)
        formatter = logging.Formatter(fmt="%(message)s")
    handler.setFormatter(formatter)
    if args.output_file:
        f_handler = logging.FileHandler(args.output_file)
        f_handler.setFormatter(formatter)
        logger.addHandler(f_handler)
    logger.addHandler(handler)

    # Read username, password, and hashes from file if it exists
    def read_from_file_if_exists(arg):
        if arg is not None and os_path.isfile(arg):
            with open(arg) as fin:
                return fin.readline().rstrip()
        else:
            return arg

    try:
        # Connection to the LDAP server using credentials provided in argument
        ldap = LdapsearchAd(
            args.ldap_server,
            args.server_port_number,
            args.ssl,
            args.domain,
            read_from_file_if_exists(args.username),
            read_from_file_if_exists(args.password),
            read_from_file_if_exists(args.hashes),
        )

        for action in actions:

            # Get the server's infos
            if action == "info":
                log_title("Server infos", 3)
                ldap.infos()

            # Check the connection and retrieve the username used for the connection
            elif action == "whoami":
                log_title('Result of "whoami" command', 3)
                ldap.whoami()

            # Raw search
            elif action == "search":
                excluded_attributes = [
                    "dSCorePropagationData",
                    "mSMQDigests",
                    "mSMQSignCertificates",
                    "userCertificate",
                    "uSNCreated",
                    "uSNChanged",
                ]
                log_title('Result of "search" command', 3)
                entries = ldap.search(
                    search_filter=args.search_filter,
                    attributes=args.search_attributes,
                    size_limit=args.size_limit,
                    search_base=args.search_base,
                )
                for entry in entries:
                    attributes = sorted([i for i in entry.keys() if i not in excluded_attributes])
                    for attribute in attributes:
                        if attribute == "userAccountControl":
                            log_info(f"|__ {attribute} = {', '.join(list_uac_flags(entry[attribute]))}")
                        elif attribute == "sAMAccountType":
                            log_info(f"|__ {attribute} = {str_samaccounttype(entry[attribute])}")
                        else:
                            log_info(f"|__ {attribute} = {entry[attribute]}")

            # CSV export
            elif action == "csv":
                attributes = args.search_attributes
                print(",".join(attributes))
                entries = ldap.search(
                    search_filter=args.search_filter,
                    attributes=args.search_attributes,
                    size_limit=args.size_limit,
                    search_base=args.search_base,
                )
                for entry in entries:
                    values = []
                    for attribute in attributes:
                        if attribute.lower() == "useraccountcontrol":
                            value = f'"{",".join(list_uac_flags(entry[attribute]))}"'
                        else:
                            value = str(entry[attribute])
                        values.append(value)
                    try:
                        print(",".join(values))
                    except TypeError as e:
                        print(f"type de values = {type(values)}")
                        print(f"values = {values}")
                        raise e

            # Get users
            elif action == "show-user":
                log_title('Result of "show-user" command', 3)
                ldap.print_users(args.search_filter, args.search_attributes, args.size_limit)

            # Get users list
            elif action == "show-user-list":
                log_title('Result of "show-user" command', 3)
                ldap.print_users_list(args.search_filter, args.search_attributes, args.size_limit)

            # Get users list of member of the specified group
            # search_filter should only contain the groupe name
            # then it is converted to a correct ldap filter
            elif action == "member-of":
                log_title('Result of "member-of" command', 3)
                if "(" in args.search_filter:
                    logging.error('You must enter a group CN instead of a full search filter. (e.g. -s "Domain admins")')
                else:
                    ldap.print_member_of(args.search_filter, args.size_limit)

            # Get the list of groups whom the user is member of
            elif action == "user-of":
                log_title('Result of "user-of" command', 3)
                ldap.print_user_of(args.search_filter, args.size_limit)

            # Get list of Foreign Security Principals added to domain local groups from external/forest trusts
            elif action == "search-foreign-security-principals":
                log_title('Result of "search-foreign-security-principals" command', 3)
                ldap.print_search_foreign_security_principals(args.size_limit)

            # Get trusts
            elif action == "trusts":
                log_title('Result of "trusts" command', 3)
                ldap.print_trusts()

            # Get password policies
            elif action == "pass-pols":
                log_title('Result of "pass-pols" command', 3)
                ldap.print_pass_pols()

            # Get admins
            elif action == "admins":
                log_title('Result of "admins" command', 3)
                ldap.print_admins()

            # Get kerberoastable users accounts
            elif action == "kerberoast":
                log_title('Result of "kerberoast" command', 3)
                ldap.print_kerberoast()

            # Get ASRepRoast user account
            elif action == "asreproast":
                log_title('Result of "asreproast" command', 3)
                ldap.print_asreqroast()

            elif action == "search-spn":
                log_title('Result of "search-spn" command', 3)
                ldap.print_search_spn(args.search_filter, args.size_limit)

            elif action == "goldenticket":
                log_title('Result of "goldenticket" command', 3)
                ldap.print_lastpwchangekrbtgt()

            elif action == "search-delegation":
                log_title('Result of "search-delegation" command', 3)
                ldap.print_search_delegation()

            elif action == "createsid":
                log_title('Result of "createsid" command', 3)
                ldap.print_creator_sid()

            # Run all checks
            elif action == "all":
                log_title("Server infos", 3)
                ldap.infos()
                log_title('Result of "trusts" command', 3)
                ldap.print_trusts()
                log_title('Result of "pass-pols" command', 3)
                ldap.print_pass_pols()
                log_title('Result of "admins" command', 3)
                ldap.print_admins()
                log_title('Result of "kerberoast" command', 3)
                ldap.print_kerberoast()
                log_title('Result of "asreqroast" command', 3)
                ldap.print_asreqroast()
                log_title('Result of "goldenticket" command', 3)
                ldap.print_lastpwchangekrbtgt()

            else:
                log_error("Error: This functionnality is not implemented yet. Please implement it now.")
    except LsaException as le:
        print(f"Error: {le}")


if __name__ == "__main__":
    main()



def c_green(message):
    """Color text for good configuration."""
    return f"\x1b[0;32;40m{message}\x1b[0m"


def c_orange(message):
    """Color text for weak configuration."""
    return f"\x1b[0;33;40m{message}\x1b[0m"


def c_red(message):
    """Color text for bad configuration."""
    return f"\x1b[0;31;40m{message}\x1b[0m"


def c_white_on_red(message):
    """Color text for very bad configuration."""
    return f"\x1b[1;37;41m{message}\x1b[0m"


def c_blue(message):
    """Color text for information."""
    return f"\x1b[0;34;40m{message}\x1b[0m"


def c_cyan(message):
    """Color text for general usefull information."""
    return f"\x1b[0;36;40m{message}\x1b[0m"


def c_purple(message):
    """Color text for abnormal behavior of the tool itself."""
    return f"\x1b[0;35;40m{message}\x1b[0m"



import codecs
import datetime
import struct

from .colors import c_green
from .colors import c_orange
from .colors import c_red
from .colors import c_white_on_red
from .colors import c_cyan
from .colors import c_purple


def str_human_date(date):
    # ldap3 version 2.6 returns a datetime.timedelta object
    if isinstance(date, datetime.timedelta):
        nb_sec = int(date.total_seconds())
    # older versions return a negative big number
    else:
        nb_sec = int((-date) / 10000000)
    if nb_sec > 60:
        nb_min = int(nb_sec / 60)
        nb_sec = nb_sec % 60
        if nb_min > 60:
            nb_hour = int(nb_min / 60)
            nb_min = nb_min % 60
            if nb_hour > 24:
                nb_day = int(nb_hour / 24)
                nb_hour = nb_hour % 24
                return f"{nb_day} days, {nb_hour} hours, {nb_min} minutes, {nb_sec} seconds"
            return f"{nb_hour} hours, {nb_min} minutes, {nb_sec} seconds"
        return f"{nb_min} minutes, {nb_sec} seconds"
    return f"{nb_sec} seconds"


def str_functionality_level(num):
    """Return the functionality level as described at:
    <https://msdn.microsoft.com/en-us/library/cc223274.aspx>
    Note: it is the same for forest, domain, and domain controller."""
    n = int(num)
    func_levels = [
        c_white_on_red("Windows 2000"),
        c_white_on_red("Windows 2003 with mixed domains"),
        c_white_on_red("Windows 2003"),
        c_red("Windows 2008"),
        c_red("Windows 2008 R2"),
        c_orange("Windows 2012"),
        c_orange("Windows 2012 R2"),
        c_green("Windows 2016"),
    ]
    if 0 <= n < len(func_levels):
        return func_levels[n]
    else:
        return f"Not known, update this script. (value = {num})"


def is_flag_in_uac(flag, uac):
    """Return a list of property flags as described at:
    <https://support.microsoft.com/en-gb/help/305144/how-to-use-the-useraccountcontrol-flags-to-manipulate-user-account-pro>
    """
    flags = {
        "SCRIPT": 0x1,
        "ACCOUNTDISABLE": 0x2,
        "HOMEDIR_REQUIRED": 0x8,
        "LOCKOUT": 0x10,
        "PASSWD_NOTREQD": 0x20,
        "PASSWD_CANT_CHANGE": 0x40,
        "ENCRYPTED_TEXT_PWD_ALLOWED": 0x80,
        "TEMP_DUPLICATE_ACCOUNT": 0x100,
        "NORMAL_ACCOUNT": 0x200,
        "INTERDOMAIN_TRUST_ACCOUNT": 0x800,
        "WORKSTATION_TRUST_ACCOUNT": 0x1000,
        "SERVER_TRUST_ACCOUNT": 0x2000,
        "DONT_EXPIRE_PASSWORD": 0x10000,
        "MNS_LOGON_ACCOUNT": 0x20000,
        "SMARTCARD_REQUIRED": 0x40000,
        "TRUSTED_FOR_DELEGATION": 0x80000,
        "NOT_DELEGATED": 0x100000,
        "USE_DES_KEY_ONLY": 0x200000,
        "DONT_REQ_PREAUTH": 0x400000,
        "PASSWORD_EXPIRED": 0x800000,
        "TRUSTED_TO_AUTH_FOR_DELEGATION": 0x1000000,
        "PARTIAL_SECRETS_ACCOUNT": 0x04000000,
    }
    return uac & flags[flag] > 0


def list_uac_flags(uac):
    """Return a list of property flags as described at:
    <https://support.microsoft.com/en-gb/help/305144/how-to-use-the-useraccountcontrol-flags-to-manipulate-user-account-pro>
    """
    flags = []
    if uac & 0x1 > 0:
        flags.append("SCRIPT")
    if uac & 0x2 > 0:
        flags.append("ACCOUNTDISABLE")
    if uac & 0x8 > 0:
        flags.append("HOMEDIR_REQUIRED")
    if uac & 0x10 > 0:
        flags.append("LOCKOUT")
    if uac & 0x20 > 0:
        flags.append("PASSWD_NOTREQD")
    if uac & 0x40 > 0:
        flags.append("PASSWD_CANT_CHANGE")
    if uac & 0x80 > 0:
        flags.append("ENCRYPTED_TEXT_PWD_ALLOWED")
    if uac & 0x100 > 0:
        flags.append("TEMP_DUPLICATE_ACCOUNT")
    if uac & 0x200 > 0:
        flags.append("NORMAL_ACCOUNT")
    if uac & 0x800 > 0:
        flags.append("INTERDOMAIN_TRUST_ACCOUNT")
    if uac & 0x1000 > 0:
        flags.append("WORKSTATION_TRUST_ACCOUNT")
    if uac & 0x2000 > 0:
        flags.append("SERVER_TRUST_ACCOUNT")
    if uac & 0x10000 > 0:
        flags.append("DONT_EXPIRE_PASSWORD")
    if uac & 0x20000 > 0:
        flags.append("MNS_LOGON_ACCOUNT")
    if uac & 0x40000 > 0:
        flags.append("SMARTCARD_REQUIRED")
    if uac & 0x80000 > 0:
        flags.append("TRUSTED_FOR_DELEGATION")
    if uac & 0x100000 > 0:
        flags.append("NOT_DELEGATED")
    if uac & 0x200000 > 0:
        flags.append("USE_DES_KEY_ONLY")
    if uac & 0x400000 > 0:
        flags.append("DONT_REQ_PREAUTH")
    if uac & 0x800000 > 0:
        flags.append("PASSWORD_EXPIRED")
    if uac & 0x1000000 > 0:
        flags.append("TRUSTED_TO_AUTH_FOR_DELEGATION")
    if uac & 0x04000000 > 0:
        flags.append("PARTIAL_SECRETS_ACCOUNT")
    return flags


def list_uac_colored_flags(uac):
    """Return a list of property flags as described at:
    <https://support.microsoft.com/en-gb/help/305144/how-to-use-the-useraccountcontrol-flags-to-manipulate-user-account-pro>
    """
    flags = []
    if uac & 0x1 > 0:
        flags.append("SCRIPT")
    if uac & 0x2 > 0:
        flags.append(c_cyan("ACCOUNTDISABLE"))
    if uac & 0x8 > 0:
        flags.append("HOMEDIR_REQUIRED")
    if uac & 0x10 > 0:
        flags.append(c_orange("LOCKOUT"))
    if uac & 0x20 > 0:
        flags.append(c_white_on_red("PASSWD_NOTREQD"))
    if uac & 0x40 > 0:
        flags.append(c_red("PASSWD_CANT_CHANGE"))
    if uac & 0x80 > 0:
        flags.append(c_white_on_red("ENCRYPTED_TEXT_PWD_ALLOWED"))
    if uac & 0x100 > 0:
        flags.append("TEMP_DUPLICATE_ACCOUNT")
    if uac & 0x200 > 0:
        flags.append("NORMAL_ACCOUNT")
    if uac & 0x800 > 0:
        flags.append(c_cyan("INTERDOMAIN_TRUST_ACCOUNT"))
    if uac & 0x1000 > 0:
        flags.append(c_cyan("WORKSTATION_TRUST_ACCOUNT"))
    if uac & 0x2000 > 0:
        flags.append(c_cyan("SERVER_TRUST_ACCOUNT"))
    if uac & 0x10000 > 0:
        flags.append(c_red("DONT_EXPIRE_PASSWORD"))
    if uac & 0x20000 > 0:
        flags.append("MNS_LOGON_ACCOUNT")
    if uac & 0x40000 > 0:
        flags.append("SMARTCARD_REQUIRED")
    if uac & 0x80000 > 0:
        flags.append(c_orange("TRUSTED_FOR_DELEGATION"))
    if uac & 0x100000 > 0:
        flags.append("NOT_DELEGATED")
    if uac & 0x200000 > 0:
        flags.append(c_red("USE_DES_KEY_ONLY"))
    if uac & 0x400000 > 0:
        flags.append(c_red("DONT_REQ_PREAUTH"))
    if uac & 0x800000 > 0:
        flags.append(c_cyan("PASSWORD_EXPIRED"))
    if uac & 0x1000000 > 0:
        flags.append(c_orange("TRUSTED_TO_AUTH_FOR_DELEGATION"))
    if uac & 0x04000000 > 0:
        flags.append("PARTIAL_SECRETS_ACCOUNT")
    return flags


def str_samaccounttype(sat):
    """Return the SAM-Account-Type as described at:
    <https://docs.microsoft.com/en-us/windows/desktop/adschema/a-samaccounttype>"""
    if sat == 0x0:
        return "SAM_DOMAIN_OBJECT"
    elif sat == 0x10000000:
        return "SAM_GROUP_OBJECT"
    elif sat == 0x10000001:
        return "SAM_NON_SECURITY_GROUP_OBJECT"
    elif sat == 0x20000000:
        return "SAM_ALIAS_OBJECT"
    elif sat == 0x20000001:
        return "SAM_NON_SECURITY_ALIAS_OBJECT"
    elif sat == 0x30000000:
        return "SAM_USER_OBJECT"
    elif sat == 0x30000000:
        return "SAM_NORMAL_USER_ACCOUNT"
    elif sat == 0x30000001:
        return "SAM_MACHINE_ACCOUNT"
    elif sat == 0x30000002:
        return "SAM_TRUST_ACCOUNT"
    elif sat == 0x40000000:
        return "SAM_APP_BASIC_GROUP"
    elif sat == 0x40000001:
        return "SAM_APP_QUERY_GROUP"
    elif sat == 0x7FFFFFFF:
        return "SAM_ACCOUNT_TYPE_MAX"
    else:
        return c_purple("Error: unknown value")


def str_object_type(entry):
    if "sAMAccountType" in entry:
        sat = entry["sAMAccountType"]
        if sat == 0x0:
            return "domain"
        elif sat == 0x10000000:
            return "group"
        elif sat == 0x20000000:
            return "alias"
        elif sat == 0x30000000:
            return "user"
        elif sat == 0x30000001:
            return "computer"
        else:
            return c_purple(f"sAMAccountType = {sat}. Please complete this script.")
    else:
        return c_purple("Unable to find correct type (sAMAccountType not present).")


def list_trust_type(trust_type):
    """Return the trust type as defined here: <https://msdn.microsoft.com/en-us/library/cc223771.aspx>"""
    if trust_type == 1:
        return "The trusted domain is a Windows domain not running Active Directory."
    elif trust_type == 2:
        return "The trusted domain is a Windows domain running Active Directory."
    elif trust_type == 3:
        return "The trusted domain is running a non-Windows, RFC4120-compliant Kerberos distribution."
    elif trust_type == 4:
        return "Historical reference; this value is not used in Windows."
    else:
        return "Error: unknown value."


def list_trust_direction(trust_direction):
    """Return the trust direction as defined here: <https://msdn.microsoft.com/en-us/library/cc223768.aspx>"""
    if trust_direction == 0:
        return "Disabled"
    elif trust_direction == 1:
        return c_green("Outbound")
    elif trust_direction == 2:
        return c_green("Inbound")
    elif trust_direction == 3:
        return c_cyan("Bidirectional")
    else:
        return c_purple("Error: unknown value.")


def list_trust_attributes(ta):
    """Return the trust attribute flags as defined here: <https://msdn.microsoft.com/en-us/library/cc223779.aspx>"""
    flags = []
    if ta & 0x1 > 0:
        flags.append("TRUST_ATTRIBUTE_NON_TRANSITIVE")
    if ta & 0x2 > 0:
        flags.append("TRUST_ATTRIBUTE_UPLEVEL_ONLY")
    if ta & 0x4 > 0:
        flags.append("TRUST_ATTRIBUTE_QUARANTINED_DOMAIN")
    if ta & 0x8 > 0:
        flags.append("TRUST_ATTRIBUTE_FOREST_TRANSITIVE")
    if ta & 0x10 > 0:
        flags.append("TRUST_ATTRIBUTE_CROSS_ORGANIZATION")
    if ta & 0x20 > 0:
        flags.append("TRUST_ATTRIBUTE_WITHIN_FOREST")
    if ta & 0x40 > 0:
        flags.append("TRUST_ATTRIBUTE_TREAT_AS_EXTERNAL")
    if ta & 0x80 > 0:
        flags.append("TRUST_ATTRIBUTE_USES_RC4_ENCRYPTION")
    if ta & 0x200 > 0:
        flags.append("TRUST_ATTRIBUTE_CROSS_ORGANIZATION_NO_TGT_DELEGATION")
    if ta & 0x400 > 0:
        flags.append("TRUST_ATTRIBUTE_PIM_TRUST")
    return flags


def convert_sid_to_string(hexstr):
    value = codecs.encode(bytes(hexstr[0]), "hex")
    init_value = bytes(hexstr[0])
    value = "S-1-5"
    for i in range(8, len(init_value), 4):
        value += f'-{str(struct.unpack("<I", init_value[i:i+4])[0])}'
    return value



from .example import add_one
from .ldapsearchad import LdapsearchAd


# version number is just the date of the release.
# with format: YYYY.MM.DD with zero so setuptools is happy
VERSION = "2024.12.10"


def version():
    return VERSION



from setuptools import setup, find_packages

setup(
    name="ldapsearch-ad",
    version="2022.8.24",
    packages=find_packages(),
    scripts=["ldapsearch-ad.py"],
    install_requires=["ldap3", "pycryptodome"],
    description="Python3 script to quickly get various information from a domain controller through his LDAP service.",
    url="https://github.com/yaap7ldapsearch-ad",
    license="GPLv3",
)
